package de.ozgcloud.nachrichten.postfach.bayernid;

import java.time.ZonedDateTime;
import java.util.GregorianCalendar;
import java.util.Objects;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.mapstruct.Condition;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.StringBasedIdentifier;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcAbsender;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcAttachmentMetadata;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcBayernIdMessageMetadata;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcSendBayernIdMessageRequest;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.GrpcSendBayernIdMessageResponse;

@Mapper(unmappedTargetPolicy = ReportingPolicy.IGNORE)
interface BayernIdPostfachNachrichtMapper {

	String MESSAGE_LEVEL = "LEVEL_1";

	@Mapping(target = "attachments", ignore = true)
	@Mapping(target = "messageMetadata", expression = "java(toBayernIdMessageMetadata(nachricht, absender))")
	GrpcSendBayernIdMessageRequest toSendBayernIdMessageMetadataRequest(PostfachNachricht nachricht, Absender absender);

	@Mapping(target = "messageId", source = "nachricht.messageId")
	@Mapping(target = "createdAt", expression = "java(convertZonedDateTime(nachricht.getCreatedAt()))")
	@Mapping(target = "subject", source = "nachricht.subject")
	@Mapping(target = "storkQaaLevel", constant = MESSAGE_LEVEL)
	@Mapping(target = "vorgangId", source = "nachricht.vorgangId")
	@Mapping(target = "text", source = "nachricht.mailBody")
	@Mapping(target = "empfaenger.postkorbId", source = "nachricht.postfachAddress")
	GrpcBayernIdMessageMetadata toBayernIdMessageMetadata(PostfachNachricht nachricht, Absender absender);

	@Mapping(target = "gemeindeschluessel", source = "gemeindeSchluessel")
	GrpcAbsender toAbsender(Absender absender);

	@Mapping(target = "messageMetadata", ignore = true)
	@Mapping(target = "attachments.content", ignore = true)
	@Mapping(target = "attachments.attachmentMetadata", expression = "java(toAttachmentMetadata(bayernIdAttachment))")
	GrpcSendBayernIdMessageRequest toSendBayernIdAttachmentsMetadataRequest(BayernIdAttachment bayernIdAttachment);

	@Mapping(target = "fileType", source = "contentType")
	GrpcAttachmentMetadata toAttachmentMetadata(BayernIdAttachment attachment);

	default String convertZonedDateTime(ZonedDateTime zonedDateTime) {
		try {
			return DatatypeFactory.newInstance().newXMLGregorianCalendar(GregorianCalendar.from(zonedDateTime)).toString();
		} catch (DatatypeConfigurationException e) {
			throw new TechnicalException("Error creating XMLGregorianCalendar which should not happen in this case.", e);
		}
	}

	BayernIdResponse fromSendBayernIdMessageResponse(GrpcSendBayernIdMessageResponse response);

	default String toPostkorbId(PostfachAddress postfachAddress) {
		return ((StringBasedIdentifier) postfachAddress.getIdentifier()).getPostfachId();
	}

	@Condition
	default boolean nonNull(String value) {
		return Objects.nonNull(value);
	}

}
